"use strict";
const utils = require('./utils');
let Router = require('koa-router');
(function (HttpMethod) {
    HttpMethod[HttpMethod["HEAD"] = 0] = "HEAD";
    HttpMethod[HttpMethod["OPTIONS"] = 1] = "OPTIONS";
    HttpMethod[HttpMethod["GET"] = 2] = "GET";
    HttpMethod[HttpMethod["PUT"] = 3] = "PUT";
    HttpMethod[HttpMethod["PATCH"] = 4] = "PATCH";
    HttpMethod[HttpMethod["POST"] = 5] = "POST";
    HttpMethod[HttpMethod["DELETE"] = 6] = "DELETE";
})(exports.HttpMethod || (exports.HttpMethod = {}));
var HttpMethod = exports.HttpMethod;
function route(path, method, ...middleware) {
    return (target, key, descriptor) => {
        utils.assert(!!path, '@route should have at least "path" argument');
        let targetProto = target.prototype || target;
        if (!targetProto.router) {
            targetProto.router = new Router();
        }
        if (typeof target === 'function' && key === undefined && descriptor === undefined) {
            utils.assert(!method, '@route on Type should not have "method"');
            targetProto.router.prefix(path);
            if (middleware.length > 0) {
                targetProto.router.use(...middleware);
            }
            return;
        }
        utils.assert(!!method, '@route on method should have "method" as second argument');
        switch (method) {
            case HttpMethod.HEAD:
                targetProto.router.head(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.OPTIONS:
                targetProto.router.options(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.GET:
                targetProto.router.get(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.PUT:
                targetProto.router.put(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.PATCH:
                targetProto.router.patch(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.POST:
                targetProto.router.post(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.DELETE:
                targetProto.router.delete(path, ...middleware, descriptor.value);
                break;
            default:
                throw new Error('@route decorator "method" is not valid');
        }
    };
}
exports.route = route;
//# sourceMappingURL=index.js.map